@@ -32,13 +32,16 @@
 #include "6model/reprs/CStr.h"
 #include "6model/reprs/CArray.h"
 #include "6model/reprs/CStruct.h"
+#include "6model/reprs/CUnion.h"
 #include "6model/reprs/ReentrantMutex.h"
 #include "6model/reprs/ConditionVariable.h"
 #include "6model/reprs/Semaphore.h"
 #include "6model/reprs/ConcBlockingQueue.h"
 #include "6model/reprs/MVMAsyncTask.h"
 #include "6model/reprs/MVMNull.h"
+#include "6model/reprs/CPPStruct.h"
 #include "6model/reprs/NativeRef.h"
+#include "6model/reprs/MultiDimArray.h"
 
 /* REPR related functions. */
 void MVM_repr_initialize_registry(MVMThreadContext *tc);
@@ -88,8 +91,11 @@ const MVMREPROps * MVM_repr_get_by_name(MVMThreadContext *tc, MVMString *name);
 #define MVM_REPR_ID_MVMAsyncTask            37
 #define MVM_REPR_ID_MVMNull                 38
 #define MVM_REPR_ID_NativeRef               39
+#define MVM_REPR_ID_MVMCUnion               40
+#define MVM_REPR_ID_MultiDimArray           41
+#define MVM_REPR_ID_MVMCPPStruct            42
 
-#define MVM_REPR_CORE_COUNT                 40
+#define MVM_REPR_CORE_COUNT                 43
 #define MVM_REPR_MAX_COUNT                  64
 
 /* Default attribute functions for a REPR that lacks them. */
@@ -110,6 +116,8 @@ const MVMREPROps * MVM_repr_get_by_name(MVMThreadContext *tc, MVMString *name);
     MVM_REPR_DEFAULT_GET_NUM, \
     MVM_REPR_DEFAULT_SET_STR, \
     MVM_REPR_DEFAULT_GET_STR, \
+    MVM_REPR_DEFAULT_SET_UINT, \
+    MVM_REPR_DEFAULT_GET_UINT, \
     MVM_REPR_DEFAULT_GET_BOXED_REF \
 }
 
@@ -119,12 +127,15 @@ const MVMREPROps * MVM_repr_get_by_name(MVMThreadContext *tc, MVMString *name);
     MVM_REPR_DEFAULT_AT_POS, \
     MVM_REPR_DEFAULT_BIND_POS, \
     MVM_REPR_DEFAULT_SET_ELEMS, \
-    MVM_REPR_DEFAULT_EXISTS_POS, \
     MVM_REPR_DEFAULT_PUSH, \
     MVM_REPR_DEFAULT_POP, \
     MVM_REPR_DEFAULT_UNSHIFT, \
     MVM_REPR_DEFAULT_SHIFT, \
     MVM_REPR_DEFAULT_SPLICE, \
+    MVM_REPR_DEFAULT_AT_POS_MULTIDIM, \
+    MVM_REPR_DEFAULT_BIND_POS_MULTIDIM, \
+    MVM_REPR_DEFAULT_DIMENSIONS, \
+    MVM_REPR_DEFAULT_SET_DIMENSIONS, \
     MVM_REPR_DEFAULT_GET_ELEM_STORAGE_SPEC \
 }
 
@@ -163,17 +174,22 @@ void MVM_REPR_DEFAULT_SET_NUM(MVMThreadContext *tc, MVMSTable *st, MVMObject *ro
 MVMnum64 MVM_REPR_DEFAULT_GET_NUM(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data);
 void MVM_REPR_DEFAULT_SET_STR(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMString *value);
 MVMString * MVM_REPR_DEFAULT_GET_STR(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data);
+void MVM_REPR_DEFAULT_SET_UINT(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMuint64 value);
+MVMuint64 MVM_REPR_DEFAULT_GET_UINT(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data);
 void * MVM_REPR_DEFAULT_GET_BOXED_REF(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMuint32 repr_id);
 
 /* Default positional indexing REPR function for a REPR that lacks it. */
 void MVM_REPR_DEFAULT_AT_POS(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value, MVMuint16 kind);
 void MVM_REPR_DEFAULT_BIND_POS(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister value, MVMuint16 kind);
 void MVM_REPR_DEFAULT_SET_ELEMS(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMuint64 count);
-MVMint64 MVM_REPR_DEFAULT_EXISTS_POS(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index);
 void MVM_REPR_DEFAULT_PUSH(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMRegister value, MVMuint16 kind);
 void MVM_REPR_DEFAULT_POP(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMRegister *value, MVMuint16 kind);
 void MVM_REPR_DEFAULT_UNSHIFT(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMRegister value, MVMuint16 kind);
 void MVM_REPR_DEFAULT_SHIFT(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMRegister *value, MVMuint16 kind);
+void MVM_REPR_DEFAULT_AT_POS_MULTIDIM(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 num_indices, MVMint64 *indices, MVMRegister *value, MVMuint16 kind);
+void MVM_REPR_DEFAULT_BIND_POS_MULTIDIM(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 num_indices, MVMint64 *indices, MVMRegister value, MVMuint16 kind);
+void MVM_REPR_DEFAULT_DIMENSIONS(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 *num_dimensions, MVMint64 **dimensions);
+void MVM_REPR_DEFAULT_SET_DIMENSIONS(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 num_dimensions, MVMint64 *dimensions);
 MVMStorageSpec MVM_REPR_DEFAULT_GET_ELEM_STORAGE_SPEC(MVMThreadContext *tc, MVMSTable *st);
 
 /* Default associative indexing REPR function for a REPR that lacks it. */
