@@ -7,14 +7,15 @@
  */
 
 /* Version information. */
-#define MVM_VERSION "2015.03"
+#define MVM_VERSION "2015.12"
 #define MVM_VERSION_MAJOR 2015
-#define MVM_VERSION_MINOR 03
+#define MVM_VERSION_MINOR 12
 #define MVM_VERSION_PATCH 0
 
 /* Various compiler-specific pragmii. */
 #define MVM_NO_RETURN 
 #define MVM_NO_RETURN_GCC __attribute__((noreturn))
+#define MVM_FORMAT(X, Y, Z) __attribute__((format(X, Y, Z)))
 
 /* DLL interface definitions. */
 #define MVM_DLL_IMPORT 
@@ -23,28 +24,35 @@
 
 /* Set in case of big-endian sytems.
  * The default byte order is little-endian. */
-#if 0
-#define MVM_BIGENDIAN 0
+#if 1
+#define MVM_BIGENDIAN 1
 #endif
 
-/* Readline detection */
-#if 0
-#define MVM_HAS_READLINE 0
+/* pthread_yield() detection */
+#if 1
+#define MVM_HAS_PTHREAD_YIELD 1
 #endif
 
 /* How this compiler does static inline functions. */
 #define MVM_STATIC_INLINE static __inline__
 
-#if 1
+#if 0
 #define MVM_CAN_UNALIGNED_INT32
 #endif
 
-#if 1
+#if 0
 #define MVM_CAN_UNALIGNED_INT64
 #endif
 
-#if 1
+#if 0
 #define MVM_CAN_UNALIGNED_NUM64
 #endif
 
 #define MVM_PTR_SIZE 8
+
+#if 1
+#define MVM_BOOL _Bool
+#endif
+
+/* Should we translate \n to \r\n on output? */
+#define MVM_TRANSLATE_NEWLINE_OUTPUT 0
