@@ -45,11 +45,13 @@ struct MVMStringConsts {
     MVMString *name;
     MVMString *attribute;
     MVMString *of;
+    MVMString *rw;
     MVMString *type;
     MVMString *typeobj;
     MVMString *free_str;
     MVMString *callback_args;
     MVMString *encoding;
+    MVMString *inlined;
     MVMString *repr;
     MVMString *anon;
     MVMString *P6opaque;
@@ -70,6 +72,9 @@ struct MVMStringConsts {
     MVMString *refkind;
     MVMString *positional;
     MVMString *lexical;
+    MVMString *dimensions;
+    MVMString *ready;
+    MVMString *multidim;
 };
 
 /* An entry in the representations registry. */
@@ -115,9 +120,11 @@ struct MVMInstance {
      * marked. */
     MVMThreadContext *event_loop_thread;
     uv_mutex_t        mutex_event_loop_start;
+    uv_sem_t          sem_event_loop_started;
     MVMObject        *event_loop_todo_queue;
     MVMObject        *event_loop_cancel_queue;
     MVMObject        *event_loop_active;
+    uv_async_t       *event_loop_wakeup;
 
     /* The VM null object. */
     MVMObject *VMNull;
@@ -159,8 +166,10 @@ struct MVMInstance {
     /* int -> str cache */
     MVMString **int_to_str_cache;
 
-    /* Specialization installation mutex (global, as it's low contention, so
-     * no real motivation to have it more fine-grained at present). */
+    /* Multi-dispatch cache and specialization installation mutexes
+     * (global, as the additions are quite low contention, so no
+     * real motivation to have it more fine-grained at present). */
+    uv_mutex_t mutex_multi_cache_add;
     uv_mutex_t mutex_spesh_install;
 
     /* Log file for specializations, if we're to log them. */
@@ -168,11 +177,13 @@ struct MVMInstance {
 
     /* Log file for dynamic var performance, if we're to log it. */
     FILE *dynvar_log_fh;
+    MVMint64 dynvar_log_lasttime;
 
     /* Flag for if spesh (and certain spesh features) are enabled. */
     MVMint8 spesh_enabled;
     MVMint8 spesh_inline_enabled;
     MVMint8 spesh_osr_enabled;
+    MVMint8 spesh_nodelay;
 
     /* Flag for if NFA debugging is enabled. */
     MVMint8 nfa_debug_enabled;
@@ -185,6 +196,10 @@ struct MVMInstance {
 
     /* Directory name for JIT bytecode dumps */
     char *jit_bytecode_dir;
+    /* File for map of frame information for bytecode dumps */
+    FILE *jit_bytecode_map;
+    /* sequence number for JIT compiled frames */
+    AO_t  jit_seq_nr;
 
     /* Number of representations registered so far. */
     MVMuint32 num_reprs;
@@ -260,9 +275,9 @@ struct MVMInstance {
     MVMIntConstCache    *int_const_cache;
     uv_mutex_t mutex_int_const_cache;
 
-    /* Atomically-incremented counter of newly invoked frames,
-     * so each can obtain an index into each threadcontext's pool table */
-    AO_t num_frame_pools;
+    /* Atomically-incremented counter of newly invoked frames, used for
+     * lexotic caching. */
+    AO_t num_frames_run;
 
     /* Hash of compiler objects keyed by name */
     MVMObject          *compiler_registry;
@@ -314,6 +329,9 @@ struct MVMInstance {
     /* Fixed size allocator. */
     MVMFixedSizeAlloc *fsa;
 
+    /* Normal Form Grapheme state (synthetics table, lookup, etc.). */
+    MVMNFGState *nfg;
+
     /* Next type cache ID, to go in STable. */
     AO_t cur_type_cache_id;
 
@@ -326,4 +344,19 @@ struct MVMInstance {
 
     /* Whether profiling is turned on or not. */
     MVMuint32 profiling;
+
+    /* Whether cross-thread write logging is turned on or not, and an output
+     * mutex for it. */
+    MVMuint32  cross_thread_write_logging;
+    MVMuint32  cross_thread_write_logging_include_locked;
+    uv_mutex_t mutex_cross_thread_write_logging;
+
+    /* Cached backend config hash. */
+    MVMObject *cached_backend_config;
 };
+
+/* Returns a true value if we have created user threads (and so are running a
+ * multi-threaded application). */
+MVM_STATIC_INLINE MVMint32 MVM_instance_have_user_threads(MVMThreadContext *tc) {
+    return tc->instance->next_user_thread_id != 2;
+}
