@@ -16,13 +16,13 @@ union MVMRegister {
     MVMObject         *o;
     MVMString *s;
     MVMint8            i8;
-    MVMuint8           ui8;
+    MVMuint8           u8;
     MVMint16           i16;
-    MVMuint16          ui16;
+    MVMuint16          u16;
     MVMint32           i32;
-    MVMuint32          ui32;
+    MVMuint32          u32;
     MVMint64           i64;
-    MVMuint64          ui64;
+    MVMuint64          u64;
     MVMnum32           n32;
     MVMnum64           n64;
 };
@@ -40,7 +40,7 @@ struct MVMOpInfo {
     MVMuint16   opcode;
     const char *name;
     char        mark[2];
-    MVMuint8    num_operands;
+    MVMuint16   num_operands;
     MVMuint8    pure;
     MVMuint8    deopt_point;
     MVMuint8    no_inline;
@@ -65,6 +65,10 @@ struct MVMOpInfo {
 #define MVM_reg_num64           6
 #define MVM_reg_str             7
 #define MVM_reg_obj             8
+#define MVM_reg_uint8           17
+#define MVM_reg_uint16          18
+#define MVM_reg_uint32          19
+#define MVM_reg_uint64          20
 
 /* Operand data types. */
 #define MVM_operand_int8        (MVM_reg_int8 << 3)
@@ -80,6 +84,10 @@ struct MVMOpInfo {
 #define MVM_operand_coderef     (12 << 3)
 #define MVM_operand_callsite    (13 << 3)
 #define MVM_operand_spesh_slot  (16 << 3)
+#define MVM_operand_uint8       (MVM_reg_uint8 << 3)
+#define MVM_operand_uint16      (MVM_reg_uint16 << 3)
+#define MVM_operand_uint32      (MVM_reg_uint32 << 3)
+#define MVM_operand_uint64      (MVM_reg_uint64 << 3)
 #define MVM_operand_type_mask   (31 << 3)
 
 /* Functions. */
