@@ -10,6 +10,9 @@
 /* Standard integer types. */
 #include <platform/inttypes.h>
 
+/* platform-specific setjmp override */
+#include <platform/setjmp.h>
+
 /* stuff for uthash */
 #define uthash_fatal(msg) MVM_exception_throw_adhoc(tc, "internal hash error: " msg)
 
@@ -21,12 +24,17 @@
 #include <uv.h>
 
 /* libatomic_ops */
+#define AO_REQUIRE_CAS
 #include <atomic_ops.h>
 
-/* dynload/dyncall/dyncallback */
+/* libffi or dynload/dyncall/dyncallback */
+#ifdef HAVE_LIBFFI
+#include <ffi.h>
+#else
 #include <dynload.h>
 #include <dyncall.h>
 #include <dyncall_callback.h>
+#endif
 
 /* forward declarations */
 #include "types.h"
@@ -55,6 +63,9 @@ typedef double   MVMnum64;
 #define ALIGNOF(t) ((char *)(&((struct { char c; t _h; } *)0)->_h) - (char *)0)
 #endif
 
+#define MVM_ALIGN_SECTION_MASK ((MVMuint32)ALIGNOF(MVMint64) - 1)
+#define MVM_ALIGN_SECTION(offset) (((offset) + (MVM_ALIGN_SECTION_MASK)) & ~(MVM_ALIGN_SECTION_MASK))
+
 #if defined MVM_BUILD_SHARED
 #  define MVM_PUBLIC  MVM_DLL_EXPORT
 #  define MVM_PRIVATE MVM_DLL_LOCAL
@@ -99,9 +110,14 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "core/loadbytecode.h"
 #include "math/num.h"
 #include "core/coerce.h"
-#include "core/dll.h"
 #include "core/ext.h"
+#ifdef HAVE_LIBFFI
+#include "core/nativecall_libffi.h"
+#else
+#include "core/nativecall_dyncall.h"
+#endif
 #include "core/nativecall.h"
+#include "core/dll.h"
 #include "core/continuation.h"
 #include "6model/reprs.h"
 #include "6model/reprconv.h"
@@ -110,11 +126,11 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "6model/sc.h"
 #include "6model/serialization.h"
 #include "6model/parametric.h"
+#include "gc/gen2.h"
 #include "gc/allocation.h"
 #include "gc/worklist.h"
 #include "gc/collect.h"
 #include "gc/orchestrate.h"
-#include "gc/gen2.h"
 #include "gc/roots.h"
 #include "gc/objectid.h"
 #include "gc/finalize.h"
@@ -131,10 +147,13 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "spesh/threshold.h"
 #include "spesh/inline.h"
 #include "spesh/osr.h"
+#include "strings/normalize.h"
 #include "strings/decode_stream.h"
 #include "strings/ascii.h"
 #include "strings/utf8.h"
+#include "strings/utf8_c8.h"
 #include "strings/utf16.h"
+#include "strings/nfg.h"
 #include "strings/iter.h"
 #include "strings/ops.h"
 #include "strings/unicode_gen.h"
@@ -144,8 +163,8 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "io/io.h"
 #include "io/eventloop.h"
 #include "io/syncfile.h"
-#include "io/syncpipe.h"
 #include "io/syncstream.h"
+#include "io/syncpipe.h"
 #include "io/syncsocket.h"
 #include "io/fileops.h"
 #include "io/dirops.h"
@@ -154,6 +173,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "io/filewatchers.h"
 #include "io/signals.h"
 #include "io/asyncsocket.h"
+#include "io/asyncsocketudp.h"
 #include "math/bigintops.h"
 #include "mast/driver.h"
 #include "core/intcache.h"
@@ -164,6 +184,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "profiler/instrument.h"
 #include "profiler/log.h"
 #include "profiler/profile.h"
+#include "instrument/crossthreadwrite.h"
 
 MVMObject *MVM_backend_config(MVMThreadContext *tc);
 
@@ -171,7 +192,7 @@ MVMObject *MVM_backend_config(MVMThreadContext *tc);
 MVM_PUBLIC MVMInstance * MVM_vm_create_instance(void);
 MVM_PUBLIC void MVM_vm_run_file(MVMInstance *instance, const char *filename);
 MVM_PUBLIC void MVM_vm_dump_file(MVMInstance *instance, const char *filename);
-MVM_PUBLIC void MVM_vm_exit(MVMInstance *instance);
+MVM_PUBLIC MVM_NO_RETURN void MVM_vm_exit(MVMInstance *instance) MVM_NO_RETURN_GCC;
 MVM_PUBLIC void MVM_vm_destroy_instance(MVMInstance *instance);
 MVM_PUBLIC void MVM_vm_set_clargs(MVMInstance *instance, int argc, char **argv);
 MVM_PUBLIC void MVM_vm_set_exec_name(MVMInstance *instance, const char *exec_name);
@@ -195,14 +216,6 @@ MVM_PUBLIC void MVM_vm_set_lib_path(MVMInstance *instance, int count, const char
 /* Full memory barrier. */
 #define MVM_barrier() AO_nop_full()
 
-/* Convenience shortcut for use in gc_free routines. */
-#define MVM_checked_free_null(addr) do { \
-    if ((addr)) { \
-        MVM_free((void *)(addr)); \
-        (addr) = NULL; \
-    } \
-} while (0)
-
 /* Need to use these to assign to or read from any memory locations on
  * which the other atomic operation macros are used... */
 #define MVM_store(addr, new) AO_store_full((volatile AO_t *)(addr), (AO_t)(new))
